/*
------------------------------
Onion Main Extended Menu
David Humpherys
Blur Studios
daveh@blur.com
------------------------------
*/

RCMenu OnionMoreMenu
(
	SubMenu "Manage Layers"
	(
		MenuItem miGetLayerManager "Import REZN8 Layer Manager Data"
	--	MenuItem miAlphabetize "Alphebetize Layers" enabled:false
		Seperator sep03
		MenuItem miAquireSelSet "Aquire Selection Sets" 
		MenuItem miCreateSelSet "Create Selection Sets" 
		Seperator sep02
		MenuItem miRemoveEmpty "Remove Empty Layers" 
		Seperator sep04
		MenuItem miGetOrphans "Get Orphaned Objects"
		Seperator sep05
		MenuItem miKillEverything "Kill All Onion Data"
		
	)
	SubMenu "Visibility / Selection"
	(
		MenuItem miHideAll "Hide All Layers" 
		MenuItem miUnHideAll "Unhide All Layers"  
		MenuItem miInvert "Invert Hide/Unhide"
		Seperator sep01
		MenuItem miFreezeAll "Freeze All Layers" 
		MenuItem miUnFrezzeAll "Thaw All Layers"
		MenuItem miInvertFreeze "Invert Freeze/Thaw"
	)

	Seperator sep_fin
	MenuItem miPref "Preferences"
	on miPref picked do Onion.ui.showPreferences()
	MenuItem miabout "About" 
	on miabout picked do Onion.ui.showAbout()
	

	on miGetLayerManager picked do
	(
		if (Onion.PREF_Debug_Mode)  then print ("RCMENU: miGetLayerManager picked")
		local LMVar = getappdata globaltracks[1] 9901
		if LMVar != undefined then
		(
			local LM = execute (getappdata globaltracks[1] 9901)
			local error = false
			clearlistener()
			for L = 1 to LM.Layers.count do
			(
				local newLay = Layermanager.NewLayerFromName LM.Layers[ L ].LayerName
				if newLay == undefined then
				(
					print ("could not import " + LM.Layers[ L ].LayerName + " because the layer name is not unique" )
					error = true
				)
				else
				(
					local theObjs = for obj in LM.Layers[ L ].ObjectHandleArray collect (maxops.getnodebyhandle obj)
					
					Onion.addObjectsToLayer theObjs newLay.name
					newLay.on = not LM.Layers[ L ].ishidden
					newLay.lock = LM.Layers[ L ].isfrozen						
					Onion.addLayer newLay
				)
			)
			Onion.ui.refresh()
			if error == true then messagebox "There were errors on import.  see listener for details"
		)
		else MessageBox "There is no Layer Manager Data in this scene"
	)

--	on miXRefUi picked do (
--		LM.xrefLayers = ( not (miXRefUI.checked) )
--		LM.UI.MakeRollout()
--	)

	function UpdateAllCheckBoxes CheckBoxString Value =
	(
		if (Onion.PREF_Debug_Mode)  then print ("FN: UpdateAllCheckBoxes CheckBoxString: " + CheckBoxString  + " Value: " + Value as string )
		case CheckBoxString of
		(
			"LToggle": (OnionRoll.worldToggle.checked = Value)
			"Lay_Freeze":(OnionRoll.worldFreeze.checked = Value)
		)
		
		for i = 1 to globaltracks.oniondata.GroupName.count do
		(
			local intAry = Onion.getLayersInGroup i	
			for L in intAry do --make sure the checkboxes are done 
			(
				execute ("grp" +i as string+ "." +CheckBoxString + "_" + L as string + ".checked = " + Value as string ) 
			)
		)
	)
	on miHideAll picked do
	(
		if (Onion.PREF_Debug_Mode)  then print ("RCMENU: miHideAll picked")	
		setWaitCursor()
		DisableSceneRedraw()
		hide objects
		max select none
		for L = 1 to layermanager.count do 
		(
			t = layermanager.getlayer (L-1)
			t.on = false			
		)
		---update all checkboxes
		UpdateAllCheckBoxes "LToggle" false
		EnableSceneRedraw()
		SetArrowCursor()
		redrawviews()
	)
	on miUnHideAll picked do
	(
		if (Onion.PREF_Debug_Mode)  then print ("RCMENU: miUnHideAll picked")
		setWaitCursor()
		DisableSceneRedraw()
		unhide objects

		for L = 1 to layermanager.count do 
		(
			t = layermanager.getlayer (L-1)
			t.on = true			
		)
		---update all checkboxes
		UpdateAllCheckBoxes "LToggle" true
		EnableSceneRedraw()
		SetArrowCursor()
	)
	on miInvert picked do
	(
		if (Onion.PREF_Debug_Mode)  then print ("RCMENU: miInvert picked")
		setWaitCursor()
		DisableSceneRedraw()
		for i = 1 to globaltracks.oniondata.GroupName.count do
		(
			local intAry = Onion.getLayersInGroup i
			for L = 1 to intAry.count do --make sure the checkboxes are done 
			(
				t = layermanager.getlayerFromName globaltracks.oniondata.layername[ intAry[ L ] ]
				if t.on == true then
				(
					t.on = false
					Onion.setHide L true
					execute ("grp" +i as string+ ".LToggle_" + L as string + ".checked = false" ) 
				)
				else
				(
					t.on = true
					Onion.setHide L false
					execute ("grp" +i as string+ ".LToggle_" + L as string + ".checked = true" ) 
				)
			)
		)
		EnableSceneRedraw()
		SetArrowCursor()
		redrawviews()
	)
	on miFreezeAll picked do
	(
		if (Onion.PREF_Debug_Mode)  then print ("RCMENU: miFreezeAll picked")
		setWaitCursor()
		DisableSceneRedraw()
		freeze objects
		max select none
		for L = 1 to layermanager.count do 
		(
			t = layermanager.getlayer (L-1)
			t.lock = true			
		)
		---update all checkboxes
		UpdateAllCheckBoxes "Lay_Freeze" true
		EnableSceneRedraw()
		SetArrowCursor()
		redrawviews()
	)
	on miUnFrezzeAll picked do
	(
		if (Onion.PREF_Debug_Mode)  then print ("RCMENU: miUnFrezzeAll picked")
		setWaitCursor()
		DisableSceneRedraw()
		unfreeze objects
		max select none
		for L = 1 to layermanager.count do 
		(
			t = layermanager.getlayer (L-1)
			t.lock = false			
		)
		---update all checkboxes
		UpdateAllCheckBoxes "Lay_Freeze" false
		EnableSceneRedraw()
		SetArrowCursor()
		redrawviews()
	)
	on miInvertFreeze picked do
	(
		if (Onion.PREF_Debug_Mode)  then print ("RCMENU: miInvertFreeze picked")
		setWaitCursor()
		DisableSceneRedraw()
		
		for i = 1 to globaltracks.oniondata.GroupName.count do
		(
			local intAry = Onion.getLayersInGroup i
			for L = 1 to intAry.count do --make sure the checkboxes are done 
			(
				t = layermanager.getlayerFromName globaltracks.oniondata.layername[ intAry[ L ] ]
				
				if t.lock == true then
				(
					t.lock = false
					Onion.setFreeze L false
					execute ("grp" +i as string+ ".Lay_Freeze_"+ L as string + ".checked = false" ) 
				)
				else
				(
					t.lock = true
					Onion.setFreeze L true
					execute ("grp" +i as string+ ".Lay_Freeze_" + L as string + ".checked = true" ) 
				)
			)
		)
		EnableSceneRedraw()
		SetArrowCursor()
		redrawviews()
	)
		
	on miRemoveEmpty picked do
	(
		if (Onion.PREF_Debug_Mode)  then print ("RCMENU: miRemoveEmpty picked")
		--start with 2 so i don't delete the 0 layer
		for L = LayerManager.count to 2 by -1 do
		(
			--try and delete it
			theLay = LayerManager.getlayer (L-1)
			Onion.deleteLayer theLay refresh:false
		)
		Onion.ui.refresh()
	)
	/*
	on miAlphabetize picked do
	(
		if LM.Layers.count > 1 then 
		(
			local NewLayerArray =#()
			NewLayerArray[1] = copy LM.Layers[1]
			local LayNames = #()
			local unsorted_LayNames = for L in LM.Layers collect L.LayerName
			for L = 2 to LM.Layers.count where LM.Layers[L].isgroupMember == false do append LayNames LM.Layers[L].LayerName
			sort LayNames
			local num = 2
			for L = 1 to LayNames.count do
			(
				--find the layer it corresponds with
				Ind = finditem unsorted_LayNames LayNames[L]
				if LM.Layers[Ind].isgroupHead == true then
				(
					append NewLayerArray (copy LM.Layers[Ind])
					local add = 1
					local sub = 1
					while sub == 1 and (Ind+add) < LM.Layers.count do
					(
						if LM.Layers[Ind+add].isGroupMember == false then 
						(
							sub = 0
							continue
						)
						append NewLayerArray LM.Layers[Ind+add]
						add += 1
					)
				)
				else append NewLayerArray  ( copy LM.Layers[Ind] )
				num +=1
			)
			LM.Layers = NewLayerArray
			LM.UI.MakeRollout()
		)
	)
	*/
	on miAquireSelSet picked do
	(
		if (Onion.PREF_Debug_Mode)  then print ("RCMENU: miAquireSelSet picked")
		Onion.convertSelectionSets2Layers()
	)
	on miCreateSelSet picked do
	(
		if (Onion.PREF_Debug_Mode)  then print ("RCMENU: miCreateSelSet picked")
		Onion.convertLayers2SelectionSets()
	)
	on miGetOrphans picked do
	(
		if (Onion.PREF_Debug_Mode)  then print ("RCMENU: miGetOrphans picked")
		disableSceneredraw()
		theList = #()
		for i = 1 to layermanager.count do
		(
			l = layermanager.getlayer (i-1)
			append theList l.name
		)
		for i in objects where i.layer.name != undefined do
		(
			name = i.layer.name
			if (finditem theList name) ==  0 then
			(
				--make a new one in the max system
				L = LayerManager.NewLayer()
				--add it to the First group--"main"
				L.setName name
--					Onion.addObjectsToLayer selectionsets[S] L.Name

				append theList name
			)
			--add the node to the layer..just to be sure.
			(layerManager.GetLayerFromName name).addnode i
		)
		enablesceneredraw()
		destroydialog OnionRoll
		Onion.aquireMaxLayers()
		Onion.doLayerCleanup()
		Onion.ShowUI()
		
	)
	on miKillEverything picked do
	(
		qry = QueryBox "This will remove all grouping and alternate materials. \nYour Layers will rebuild themselves. \nAre you sure you want to do this?"
		if qry do Onion.Kill()
	)
)	